/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * some public functions are defined here
 * $Created by: XIAO Tong (xiaotong@mail.neu.edu.cn) 2018-07-04
 *
 */

#include <stdio.h>
#include "XGlobal.h"
#include "XTensor.h"

#ifndef __XLINK_H__
#define __XLINK_H__

namespace nts{ // namespace nts(NiuTrans.Tensor)

/* cross reference */
struct XTensor;

/*
This defines the link among tensors in networks. XLink can be
cast as a hyperedge in a graph. when we compute on tensors, we actually create a
network where nodes are tensors and edges the connections among them. Each connection is
a hyperedge whose head is the output tensor and tails are input tensors. E.g,
c = a + b
represents a network with three nodes (a, b and c) and a hyperedge that links a and b (tails) to c (head).
 
   + (=c)
  / \
 a   b
 
for c, we have a incoming edge (a, b) -> c
for a, we also have a edge c -> a in the reverse order (in a view of acyclic directed graphs)
*/
struct XLink
{
    /* head of the hyperedge */
    XTensor * head;
    
    /* tails of the hyperedge ∂*/
    XTensor * tails[];
    
    XLink(){};
    
    ~XLink(){};
};
    
} // namespace nts(NiuTrans.Tensor)

#endif // __XLINK_H__
